@@ -2,6 +2,7 @@
 
 #include "esphome/core/component.h"
 #include "esphome/core/preferences.h"
+#include "esphome/core/helpers.h"
 #include <WiFiServer.h>
 #include <WiFiClient.h>
 
@@ -32,6 +33,8 @@ enum OTAResponseTypes {
   OTA_RESPONSE_ERROR_UNKNOWN = 255,
 };
 
+enum OTAState { OTA_COMPLETED = 0, OTA_STARTED, OTA_IN_PROGRESS, OTA_ERROR };
+
 /// OTAComponent provides a simple way to integrate Over-the-Air updates into your app using ArduinoOTA.
 class OTAComponent : public Component {
  public:
@@ -47,7 +50,11 @@ class OTAComponent : public Component {
   /// Manually set the port OTA should listen on.
   void set_port(uint16_t port);
 
-  void start_safe_mode(uint8_t num_attempts = 10, uint32_t enable_time = 120000);
+  bool should_enter_safe_mode(uint8_t num_attempts, uint32_t enable_time);
+
+#ifdef USE_OTA_STATE_CALLBACK
+  void add_on_state_callback(std::function<void(OTAState, float, uint8_t)> &&callback);
+#endif
 
   // ========== INTERNAL METHODS ==========
   // (In most use cases you won't need these)
@@ -82,6 +89,10 @@ class OTAComponent : public Component {
   uint32_t safe_mode_rtc_value_;
   uint8_t safe_mode_num_attempts_;
   ESPPreferenceObject rtc_;
+
+#ifdef USE_OTA_STATE_CALLBACK
+  CallbackManager<void(OTAState, float, uint8_t)> state_callback_{};
+#endif
 };
 
 }  // namespace ota
