@@ -6,6 +6,13 @@
 namespace esphome {
 namespace remote_base {
 
+struct RCSwitchData {
+  uint64_t code;
+  uint8_t protocol;
+
+  bool operator==(const RCSwitchData &rhs) const { return code == rhs.code && protocol == rhs.protocol; }
+};
+
 class RCSwitchBase {
  public:
   RCSwitchBase() = default;
@@ -28,6 +35,8 @@ class RCSwitchBase {
 
   bool decode(RemoteReceiveData &src, uint64_t *out_data, uint8_t *out_nbits) const;
 
+  optional<RCSwitchData> decode(RemoteReceiveData &src) const;
+
   static void simple_code_to_tristate(uint16_t code, uint8_t nbits, uint64_t *out_code);
 
   static void type_a_code(uint8_t switch_group, uint8_t switch_device, bool state, uint64_t *out_code,
@@ -51,7 +60,7 @@ class RCSwitchBase {
   bool inverted_{};
 };
 
-extern RCSwitchBase rc_switch_protocols[9];
+extern const RCSwitchBase RC_SWITCH_PROTOCOLS[9];
 
 uint64_t decode_binary_string(const std::string &data);
 
@@ -204,5 +213,7 @@ class RCSwitchDumper : public RemoteReceiverDumperBase {
   bool dump(RemoteReceiveData src) override;
 };
 
+using RCSwitchTrigger = RemoteReceiverTrigger<RCSwitchBase, RCSwitchData>;
+
 }  // namespace remote_base
 }  // namespace esphome
