@@ -30,21 +30,21 @@ struct Color {
     uint8_t raw[4];
     uint32_t raw_32;
   };
+
   inline Color() ALWAYS_INLINE : r(0), g(0), b(0), w(0) {}  // NOLINT
-  inline Color(float red, float green, float blue) ALWAYS_INLINE : r(uint8_t(red * 255)),
-                                                                   g(uint8_t(green * 255)),
-                                                                   b(uint8_t(blue * 255)),
-                                                                   w(0) {}
-  inline Color(float red, float green, float blue, float white) ALWAYS_INLINE : r(uint8_t(red * 255)),
-                                                                                g(uint8_t(green * 255)),
-                                                                                b(uint8_t(blue * 255)),
-                                                                                w(uint8_t(white * 255)) {}
+  inline Color(uint8_t red, uint8_t green, uint8_t blue) ALWAYS_INLINE : r(red), g(green), b(blue), w(0) {}
+
+  inline Color(uint8_t red, uint8_t green, uint8_t blue, uint8_t white) ALWAYS_INLINE : r(red),
+                                                                                        g(green),
+                                                                                        b(blue),
+                                                                                        w(white) {}
   inline Color(uint32_t colorcode) ALWAYS_INLINE : r((colorcode >> 16) & 0xFF),
                                                    g((colorcode >> 8) & 0xFF),
                                                    b((colorcode >> 0) & 0xFF),
                                                    w((colorcode >> 24) & 0xFF) {}
+
   inline bool is_on() ALWAYS_INLINE { return this->raw_32 != 0; }
-  inline Color &operator=(const Color &rhs) ALWAYS_INLINE {
+  inline Color &operator=(const Color &rhs) ALWAYS_INLINE {  // NOLINT
     this->r = rhs.r;
     this->g = rhs.g;
     this->b = rhs.b;
@@ -130,32 +130,21 @@ struct Color {
   }
   inline Color &operator-=(uint8_t subtract) ALWAYS_INLINE { return *this = (*this) - subtract; }
   static Color random_color() {
-    float r = float(random_uint32()) / float(UINT32_MAX);
-    float g = float(random_uint32()) / float(UINT32_MAX);
-    float b = float(random_uint32()) / float(UINT32_MAX);
-    float w = float(random_uint32()) / float(UINT32_MAX);
-    return Color(r, g, b, w);
+    uint32_t rand = random_uint32();
+    uint8_t w = rand >> 24;
+    uint8_t r = rand >> 16;
+    uint8_t g = rand >> 8;
+    uint8_t b = rand >> 0;
+    const uint16_t max_rgb = std::max(r, std::max(g, b));
+    return Color(uint8_t((uint16_t(r) * 255U / max_rgb)), uint8_t((uint16_t(g) * 255U / max_rgb)),
+                 uint8_t((uint16_t(b) * 255U / max_rgb)), w);
   }
-  Color fade_to_white(uint8_t amnt) { return Color(1, 1, 1, 1) - (*this * amnt); }
+  Color fade_to_white(uint8_t amnt) { return Color(255, 255, 255, 255) - (*this * amnt); }
   Color fade_to_black(uint8_t amnt) { return *this * amnt; }
   Color lighten(uint8_t delta) { return *this + delta; }
   Color darken(uint8_t delta) { return *this - delta; }
-
-  uint32_t to_rgb_565() const {
-    uint32_t color565 =
-        (esp_scale8(this->red, 31) << 11) | (esp_scale8(this->green, 63) << 5) | (esp_scale8(this->blue, 31) << 0);
-    return color565;
-  }
-  uint32_t to_bgr_565() const {
-    uint32_t color565 =
-        (esp_scale8(this->blue, 31) << 11) | (esp_scale8(this->green, 63) << 5) | (esp_scale8(this->red, 31) << 0);
-    return color565;
-  }
-  uint32_t to_grayscale4() const {
-    uint32_t gs4 = esp_scale8(this->white, 15);
-    return gs4;
-  }
 };
+
 static const Color COLOR_BLACK(0, 0, 0);
-static const Color COLOR_WHITE(1, 1, 1);
+static const Color COLOR_WHITE(255, 255, 255, 255);
 };  // namespace esphome
